#ifndef _emule_h
#define _emule_h

/* PC1450 - A calculator emulator
 * 1996 - 2001 Yves Maingoy
 * ymaingoy@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * Ajout du 24/06/2008 : ifndef/define/endif
 */

#include <windows.h>
#include <stdio.h>
#include <conio.h>

/*
    Definition des registres internes du Sharp
    v 0.01 04/11/1996
    NB: la RAM interne fait 96 bytes (0x60),
        mais les pointeurs P,Q, et R sont sur 7 bits,
        donc cette RAM est etendue a 128 bytes (0x80).
        Cette memoire suplementaire n'existe pas dans
        la realite et aucun acces ne devrait s'y faire.
        Son acces ne provoque aucun probleme, elle
        retourne toujours la valeur 0. Dans l'emulation,
        pour accelerer les traitements, aucun test
        de validite des pointeurs P,Q, et R n'est fait.
*/
extern int  isEmulateurMsg; // Variable globale pour les messages de l'emulateur
extern int  isExecuteON;    // Variable globale qui indique une excution en cours.
extern char EmulateurMsg[128]; // Message de l'emulateur

struct SHARP_WORDREGS {
    unsigned short JI;
    unsigned short BA;
    unsigned short X ;
    unsigned short Y ;
    unsigned short filler1[0x40-4];
    };

struct SHARP_BYTEREGS {
    unsigned char I, J;
    unsigned char A, B;
    unsigned char XL, XH;
    unsigned char YL, YH;
    unsigned char K, L;
    unsigned char M, N;
    unsigned short filler2[0x40-8];
    };


union SHARP_REGS {
    struct SHARP_WORDREGS x; /* sur 2 bytes */
    struct SHARP_BYTEREGS h; /* sur 1 byte  */
    unsigned char M[0x80];   /* tableau */
    unsigned short MW[0x40];   /* tableau */
    };

#define regI  0x00
#define regJ  0x01
#define regA  0x02
#define regB  0x03
#define regXL 0x04
#define regXH 0x05
#define regYL 0x06
#define regYH 0x07
#define regK  0x08
#define regL  0x09
#define regM  0x0A
#define regN  0x0B

#define regX  0x02
#define regY  0x03

#define regIA 0x5C
#define regIB 0x5D
#define regIF 0x5E
#define regIC 0x5F

extern unsigned int breakPC; /* interruption du debugger */
extern int  is_mode_proceed; /* !=0 si mode proceed */
extern unsigned int testKbdReady(unsigned char *charCode, unsigned char *scanCode);
extern void getKbdChar(unsigned char *charCode, unsigned char *scanCode);
extern unsigned int testKbdRdy(void);
extern char getOneChar();
extern int cycles; /* duree de l'instruction en cycles cpu */

/* fonction utilisee pour exectuer l'emulation */
extern void executeInstruction();


/* macros pour traiter les flags C et Z */
#define set_C0 (flags & 0x01)
#define set_C1 (flags | 0x10)
#define set_Z0 (flags & 0x10)
#define set_Z1 (flags | 0x01)

#define set_C0_Z1 (0x01)
#define set_C0_Z0 (0x00)
#define set_C1_Z1 (0x11)
#define set_C1_Z0 (0x10)


/* macros de test sur les flags C et Z */
#define is_C0  !(flags & 0x10)
#define is_Z0  !(flags & 0x01)
#define is_C1  (flags & 0x10)
#define is_Z1  (flags & 0x01)


/* declaration des registres */
extern union SHARP_REGS ram;
extern char flags;
extern unsigned char P;
extern unsigned char Q;
extern unsigned char R;
extern unsigned char D;
extern unsigned char IA;
extern unsigned char IB;
extern unsigned char IC;
extern unsigned char IF;
extern unsigned short DP;
extern unsigned short PC;

/* Macros d'increment / decrement des registes internes */
#define inc8bits() if((ram.M[Q]++)!=0xFF) flags = 0x00; else flags = 0x11;
#define dec8bits() if(!ram.M[Q]--) flags = 0x10; else { if(!ram.M[Q]) flags = 0x01; else flags = 0x00; }


/* Declaration des fonctions d'emulation */

/* Fonction inconnue */
extern void rz();
/* Fonctions d'increments */
extern void inci();
extern void incj();
extern void inca();
extern void incb();
extern void inck();
extern void incl();
extern void incm();
extern void incn();
extern void deci();
extern void decj();
extern void deca();
extern void decb();
extern void deck();
extern void decl();
extern void decm();
extern void decn();
extern void ix();
extern void dx();
extern void iy();
extern void dy();
extern void incp();
extern void decp();
extern void ixl();
extern void dys();
extern void dxl() ;
extern void iys() ;

/* instructions de chargement simples */
extern void ra();    /* ou CLA */
extern void ldd();
extern void ldm();
extern void ldp();
extern void ldq();
extern void ldr();
extern void lia();
extern void lib();
extern void lidl();
extern void lidp();
extern void lii();
extern void lij();
extern void lip();
extern void liq();
extern void lp();

extern void std();
extern void stp();
extern void stq();
extern void str();


/* instructions de saut */
extern void jp();
extern void jrm();
extern void jrp();
extern void jpc();
extern void jpz();
extern void jpnc();
extern void jpnz();

extern void jrcm();
extern void jrcp();
extern void jrncm();
extern void jrncp();

extern void jrzm();
extern void jrzp();
extern void jrnzm();
extern void jrnzp();

/* instruction de pile */
extern void pop();
extern void push();
extern void rtn();
extern void cal();
extern void call();
extern void leave();
extern void loop();
extern void ptj();

/* instruction NOP */
extern void nopw();
extern void nopt();
extern void wait();
extern void waiti();

/* instruction de manipulation de bit */
extern void rc();
extern void sc();
extern void sl();
extern void sr();
extern void swp();

/* instructions non documentees */
extern void reada();
extern void readm();



/* instruction qui n'existe pas
   servant a generer des interuptions
   pour l'emulateur  */
extern void intx() ;

/* instruction complexes */
extern void mvw() ;
extern void exw() ;
extern void mvb() ;
extern void exb() ;
extern void adn() ;
extern void sbn() ;
extern void adw() ;
extern void sbw() ;
extern void adb() ;
extern void sbb() ;
extern void mvwd();
extern void exwd();
extern void mvbd();
extern void exbd();
extern void srw() ;
extern void slw() ;
extern void film();
extern void fild();
extern void data();
extern void adm() ;
extern void sbm() ;
extern void anma();
extern void orma();
extern void ina() ;
extern void mvdm();
extern void mvmd();
extern void outa();
extern void outf();
extern void anim();
extern void orim();
extern void tsim();
extern void cpim();
extern void ania();
extern void oria();
extern void tsia();
extern void cpia();
extern void dtj() ;
extern void test();
extern void adim();
extern void sbim();
extern void adia();
extern void sbia();
extern void adcm();
extern void sbcm();
extern void tsma();
extern void cpma();
extern void inb() ;
extern void anid();
extern void orid();
extern void tsid();
extern void exab();
extern void exam();
extern void outb();
extern void outc();


#endif
