
/* PC1450 - A calculator emulator
 * 1996 - 2001 Yves Maingoy
 * ymaingoy@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
    memory.c
    Gestion de la memoire du Sharp
*/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "memory.h"
#include "error.h"
#include "emule.h"
#include "unassemble.h"

unsigned char KeyPort;  			/* adresse 0x7E00 du sharp */
int TestCompter0; 					/* Test 01 */
int TestCompter1; 					/* Test 02 */
int DebugMode ; 					/* != 0 si on doit passer en mode debug */
int FinPrgm;   						/* != 0 si fin du prgm */
int CLK; 							/* !=0 si CLK ON */

long Horloge512;					/* 512 ms */
long Horloge002;					/* 002 ms */
long MaxHorloge002;
long MaxHorloge512;
int isHorloge512; 					/* != 0 si 512 ms de passees */
int isHorloge002; 					/* != 0 si 002 ms de passees */

int isTouchePresente; /* !=0 si une touche a lire */
int NoROMLoaded; // erreur de chargement de la rom


unsigned char MemoryBuffer[sizebuf]; // Buffer RAM + ROM

extern void test_DP();


/* Effectue un poke correct dans la memoire. */
void pokePC(unsigned short adr, unsigned char byte)
{

    /* ROM interne */
    //if (adr < 0x2000) return;

    /* ROM externe */
    //if (adr >=0x8000) return;


    /* RAM externe */
    if (adr < 0x6000)
    {
        if (adr >= 0x5000) MemoryBuffer[adr] = byte ;
        else
            if (RamType ==16)  MemoryBuffer[adr] = byte ;
            else
                if (RamType == 8)
                {
                    if (adr >= 0x4000) MemoryBuffer[adr] = byte ;
                    else               MemoryBuffer[adr+0x2000] = byte ;
                }
                else /* (RamType == 4) */
                {
                    if (adr >= 0x4000) MemoryBuffer[adr+0x1000] = byte ;
                    else
                        if (adr >= 0x3000) MemoryBuffer[adr+0x2000] = byte ;
                        else               MemoryBuffer[adr+0x3000] = byte ;
                }
    }
    else if ( adr >=0x7000)
    {

            /* RAM interne (ecran) */
            // Elle est decomposee plusieurs zones :
            // 7000-703B ecran 1 (12 caracteres)
            // 7062-7076 ecran 2 (4  caracteres)
            // 703C,703D,707C sprites ecran

            if (adr <= 0x707C) {
                if(MemoryBuffer[adr] != byte) {
                    MemoryBuffer[adr] = byte ;
                    // On peut rafraichir l'ecran lors du prochain
                    // passage de rafraichissement
                    isScreenRefresh = 1;
                }
            }
            else
                if (adr < 0x70C0) {
                    MemoryBuffer[adr] = byte ;
                }

            // Rafraichir le second mappage pour la compatibilite
            if (adr < 0x71C0) {
                if(MemoryBuffer[adr-0x100] != byte) {
                    MemoryBuffer[adr-0x100] = byte ;
                }
            }

            /* cas special du KeyPort ! */
            if (adr == 0x7E00) KeyPort = byte;
            //else
    }
    /* sinon Memoire non mappee! */
}

/* Effectue un peek correct dans la memoire. */
unsigned char peekPC(unsigned short pc)
{
    unsigned char bytePC;
    unsigned int  adrPC;
    adrPC = pc;
    bytePC = MemoryBuffer[adrPC];



    /* ROM interne */
    if (adrPC < 0x2000) return bytePC;

    /* ROM externe */
    if (adrPC >=0x8000) return bytePC;

    /* RAM externe */
    if (adrPC < 0x6000)
        {
        if (adrPC>= 0x5000) return bytePC;
        if (RamType ==16)  return  bytePC;
        if (RamType == 8)
            {
            if (adrPC >= 0x4000) return  bytePC;
            else               return (unsigned char) MemoryBuffer[adrPC+0x2000];
            }
        else /* (RamType == 4) */
            {
            if (adrPC >= 0x4000) return (unsigned char) MemoryBuffer[adrPC+0x1000];
            if (adrPC >= 0x3000) return (unsigned char) MemoryBuffer[adrPC+0x2000];
            else               return (unsigned char) MemoryBuffer[adrPC+0x3000];
            }
        }
    else if ( adrPC >=0x7000)
        {
        /* RAM interne (ecran) */

        if (adrPC < 0x70C0) return  bytePC;
        if (adrPC < 0x7100) return (unsigned char) 0xFF;
        if (adrPC < 0x71C0) return  bytePC;
        if (adrPC < 0x7200) return (unsigned char) 0xFF;
        }
    /* Memoire non mappee! */

    return (unsigned char) (adrPC>>8);
}

/* Effectue un peek errone dans la memoire. */
unsigned char peekDP(unsigned short adr)
{


    /* ROM interne */
    if (adr < 0x2000) return (unsigned char) (adr>>8);

    /* ROM externe */
    if (adr >=0x8000) return (unsigned char) MemoryBuffer[adr];

    /* RAM externe */
    if (adr < 0x6000)
        {
        if (adr >= 0x5000) return (unsigned char) MemoryBuffer[adr];
        if (RamType ==16)  return (unsigned char) MemoryBuffer[adr];
        if (RamType == 8)
            {
            if (adr >= 0x4000) return (unsigned char) MemoryBuffer[adr];
            else               return (unsigned char) MemoryBuffer[adr+0x2000];
            }
        else /* (RamType == 4) */
            {
            if (adr >= 0x4000) return (unsigned char) MemoryBuffer[adr+0x1000];
            if (adr >= 0x3000) return (unsigned char) MemoryBuffer[adr+0x2000];
            else               return (unsigned char) MemoryBuffer[adr+0x3000];
            }
        }
    else if ( adr >=0x7000)
        {
        /* RAM interne (ecran) */
        if (adr < 0x70C0) return (unsigned char) MemoryBuffer[adr];
        if (adr < 0x7100) return (unsigned char) 0xFF;
        if (adr < 0x71C0) return (unsigned char) MemoryBuffer[adr-0x100];
        if (adr < 0x7200) return (unsigned char) 0xFF;
        }
    /* Memoire non mappee! */
    return (unsigned char) (adr>>8);
}


/* Les adresses du SC-61860 sont stoquees dans l'ordre */
/* inverse par rapport au format INTEL. */
unsigned short dpeekPC(unsigned short adr)
{
    int pRetour;
    pRetour = peekPC(adr++)<<8;
    pRetour |= peekPC(adr);
    return (unsigned short) pRetour;
}

unsigned short dpeekDP(unsigned short adr)
{
    int pRetour;
    pRetour = peekDP(adr++)<<8;
    pRetour |= peekDP(adr);
    return (unsigned short) pRetour;
}

/****************************************************************************

    FUNCTION: LoadMemory(char *FicName)

    COMMENTS: Charge la memoire (64Ko) depuis un fichier

****************************************************************************/

/* lecture de la rom et de la ram */
void LoadMemory(char *FicNameRAM)
{
    FILE *fpIn;  // fichier en entree
    char *FicNameROML = "PC1450L.ROM";
    char *FicNameROMH = "PC1450H.ROM";
    int ErrorLecture = 0;

    // Initialise les noms des fichiers au cas ou..
    ReadIni();


    // Lecture du fichier ROML
    if((fpIn=fopen(FicNameROML,"rb")) != NULL)
    {
        if( fread(MemoryBuffer,1,0x2000,fpIn) != 0x2000 )
        {
            ErrorLecture = 1;
        }

        fclose(fpIn);
    }
    else
    {
        ErrorLecture = 2;
    }

    // Lecture du fichier ROMH
    if((fpIn=fopen(FicNameROMH,"rb")) != NULL)
    {
        if( fread((char*)(MemoryBuffer+0x8000),1,0x8000,fpIn) != 0x8000 )
        {
            ErrorLecture = 5;
        }
        fclose(fpIn);
    }
    else
    {
        ErrorLecture = 6;
    }
    NoROMLoaded = ErrorLecture;

    // Lecture du fichier RAM
    if((fpIn=fopen(FicNameRAM,"rb")) != NULL)
    {
        if( fread((char*)(MemoryBuffer+0x2000),1,0x6000,fpIn) != 0x6000 )
        {
            ErrorLecture = 3;
        }
        fclose(fpIn);
    }
    else
    {
        ErrorLecture = 4;
    }

    // Lecture des registres depuis le fichier PC1450.ini
    ReadIni();

    // Si une erreur a ete rencontree, reinitialiser les variables internes
    if(ErrorLecture != 0)
    {
        PC = 0;
    }
}

/****************************************************************************

    FUNCTION: void SaveMemory(char *FicNameROM, char *FicNameRAM)

    COMMENTS: Sauve la memoire (64Ko) dans deux fichiers

****************************************************************************/

void SaveMemory( char *FicNameRAM)
{
    FILE *fpOut;  // fichier en sortie
/*
    char *FicNameROML = "PC1450L.ROM";
    char *FicNameROMH = "PC1450H.ROM";

    // Ecriture du fichier ROML
    if((fpOut=fopen(FicNameROML,"wb")) == NULL)
        error("Erreur pendant l'criture de la ROM Basse");
    if( fwrite((char*)(MemoryBuffer),1,0x2000,fpOut) != 0x2000 )
        error("Erreur pendant l'criture de la ROM Basse");
    fclose(fpOut);

    // Ecriture du fichier ROMH
    if((fpOut=fopen(FicNameROMH,"wb")) == NULL)
        error("Erreur pendant l'criture de la ROM Haute");
    if( fwrite((char*)(MemoryBuffer+0x8000),1,0x8000,fpOut) != 0x8000 )
        error("Erreur pendant l'criture de la ROM Haute");
    fclose(fpOut);
*/

    /* Ecriture du fichier RAM */
    if((fpOut=fopen(FicNameRAM,"wb")) == NULL)
        error("Erreur pendant l'criture de la RAM");
    if( fwrite((char*)(MemoryBuffer+0x2000),1,0x6000,fpOut) != 0x6000 )
        error("Erreur pendant l'criture de la RAM");
    fclose(fpOut);

    // sauvegarde des registres dans le fichier PC1450.ini
    WriteIni();
}

/****************************************************************************

    FUNCTION: initRegs(void)

    COMMENTS: initialisation des registres internes

****************************************************************************/
void initRegs(void)
{
    isScreenRefresh = 1; // rafraichir l'ecran
    BreakOn = 0;

    Horloge512 = 0;
    Horloge002 = 0;
    //MaxHorloge512 =10000; initialise dans le fichier ini
    MaxHorloge002 = 20;
    isHorloge512 = 0;
    isHorloge002 = 0;

    breakPC = 0xFFFF;
    DebugMode =0;
    FinPrgm = 0;
    CLK = 1; /* horloge ON */
}

void WriteIni(void)
{
    char bufValeur[16];
    char bufIndice[16];

    LPSTR  lpFileName=".\\Pc1450.ini";
    LPSTR  lpAppName ="Registres";
    int i;
    int Valeur;

    _itoa( RamType, bufValeur, 10 );
    WritePrivateProfileString("Ram", "Ko", bufValeur, lpFileName);


    _itoa( PC, bufValeur, 10 );
    WritePrivateProfileString(lpAppName, "PC", bufValeur, lpFileName);
    _itoa( DP, bufValeur, 10 );
    WritePrivateProfileString(lpAppName, "DP", bufValeur, lpFileName);
    _itoa( P, bufValeur, 10 );
    WritePrivateProfileString(lpAppName, "P", bufValeur, lpFileName);
    _itoa( Q, bufValeur, 10 );
    WritePrivateProfileString(lpAppName, "Q", bufValeur, lpFileName);
    _itoa( R, bufValeur, 10 );
    WritePrivateProfileString(lpAppName, "R", bufValeur, lpFileName);

    for( i=0; i<0x60; i++)
    {
        Valeur = ram.M[i] & 255;
        _itoa( Valeur, bufValeur, 10 );
        _itoa( i, bufIndice, 10);
        WritePrivateProfileString("Ram Interne", bufIndice, bufValeur, lpFileName);
    }

    _ltoa( MaxHorloge512, bufValeur, 10 );
    WritePrivateProfileString("Horloge", "MaxHorloge512", bufValeur, lpFileName);
    _ltoa( MaxHorloge002, bufValeur, 10 );
    WritePrivateProfileString("Horloge", "MaxHorloge002", bufValeur, lpFileName);
    _itoa( MaxDelay, bufValeur, 10 );
    WritePrivateProfileString("Horloge", "MaxDelay", bufValeur, lpFileName);
    WritePrivateProfileString("Fichiers", "FileRAM", pc1450mem, lpFileName);
    WritePrivateProfileString("Fichiers", "FileLOAD", FileNameLOAD, lpFileName);
    WritePrivateProfileString("Fichiers", "FileSAVE", FileNameSAVE, lpFileName);

    WritePrivateProfileString("Tools", "unassemble_file", unassemble_file, lpFileName);
	_ltoa( unassemble_start, bufValeur, 10 );
    WritePrivateProfileString("Tools", "unassemble_start", bufValeur, lpFileName);
	_ltoa( unassemble_stop, bufValeur, 10 );
    WritePrivateProfileString("Tools", "unassemble_stop", bufValeur, lpFileName);

}

void ReadIni(void)
{
    const int cbReturnBuffer =256;
    char bufValeur[256];
    char bufIndice[16];

    LPSTR  lpFileName=".\\Pc1450.ini";
    LPSTR  lpAppName ="Registres";
    int i;

    GetPrivateProfileString("Ram", "Ko", "16", bufValeur, cbReturnBuffer, lpFileName);
    RamType = atoi(bufValeur);

    GetPrivateProfileString(lpAppName, "PC", "0",bufValeur, cbReturnBuffer, lpFileName);
    PC = atoi(bufValeur);

    GetPrivateProfileString(lpAppName, "DP", "0",bufValeur, cbReturnBuffer, lpFileName);
    DP = atoi(bufValeur);

    GetPrivateProfileString(lpAppName, "P", "0",bufValeur, cbReturnBuffer, lpFileName);
    P  = atoi(bufValeur);
    GetPrivateProfileString(lpAppName, "Q", "0",bufValeur, cbReturnBuffer, lpFileName);
    Q  = atoi(bufValeur);
    GetPrivateProfileString(lpAppName, "R", "0",bufValeur, cbReturnBuffer, lpFileName);
    R  = atoi(bufValeur);


    for( i=0; i<0x60; i++)
    {
        _itoa( i, bufIndice, 10);
        GetPrivateProfileString("Ram Interne", bufIndice, "0",bufValeur, cbReturnBuffer, lpFileName);
        ram.M[i]  = atoi(bufValeur) & 255 ;
    }
    for( i=0x60; i<0x80; i++ )
    {
        ram.M[i] = 0;
    }
    // Recuperation de la valeur de l'horloge
    GetPrivateProfileString("Horloge", "MaxHorloge512", "50000",bufValeur, cbReturnBuffer, lpFileName);
    MaxHorloge512 = atol(bufValeur);
    if (MaxHorloge512<1000) MaxHorloge512 = 50000; // Petite precaution

    GetPrivateProfileString("Horloge", "MaxHorloge002", "20",bufValeur, cbReturnBuffer, lpFileName);
    MaxHorloge002 = atol(bufValeur);
    if (MaxHorloge002<1) MaxHorloge002 = 20; // Petite precaution

    // Delai d'attente entre chaque instruction
    // En cycle de base
    GetPrivateProfileString("Horloge", "MaxDelay", "0",bufValeur, cbReturnBuffer, lpFileName);
    MaxDelay = atoi(bufValeur);
    if (MaxDelay<0) MaxDelay = 0; // Petite precaution

    GetPrivateProfileString("Fichiers", "FileRAM", "PC1450.RAM",pc1450mem, cbReturnBuffer, lpFileName);
    GetPrivateProfileString("Fichiers", "FileLOAD", "exemple.bas",FileNameLOAD, cbReturnBuffer, lpFileName);
    GetPrivateProfileString("Fichiers", "FileSAVE", "exemple.bas",FileNameSAVE, cbReturnBuffer, lpFileName);


	//Desassembleur
    GetPrivateProfileString("Tools", "unassemble_file", "demo.asm",unassemble_file, cbReturnBuffer, lpFileName);
    GetPrivateProfileString("Tools", "unassemble_start", "0",bufValeur, cbReturnBuffer, lpFileName);
	unassemble_start = (unsigned short) atoi(bufValeur);
    GetPrivateProfileString("Tools", "unassemble_stop", "256",bufValeur, cbReturnBuffer, lpFileName);
	unassemble_stop = (unsigned short) atoi(bufValeur);


}

