
/* PC1450 - A calculator emulator
 * 1996 - 2001 Yves Maingoy
 * ymaingoy@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * Ajout du 06/12/2001 : gestion d'un menu unassemble
 */

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include "memory.h"
#include "error.h"

char unassemble_file[256]; // Fichier utilise pour dsassembler
unsigned short unassemble_start;        //adresse de depart
unsigned short unassemble_stop;         //adresse d'arrivee
int            unassemble_type;         //1=pret a reassembler, 0=defaut

/*
	liste des mnemoniques du processeur SC-61860

	*La table des mnemoniques est compose de 4 chaines
		qui seront ensuite regroupees.

	* Une memonique est composee de 16 bytes :
		1  byte  => Longueur de l'instruction en byte
		1  byte (espace)
		12 bytes => Mnemonique (string_+ \0)
		1  byte  => Numero de Mnemo
		1  byte (espace)

	* signification des caracteres
		'n' indique un octet
		'&' indique une adresse
		'+' indique un deplacement
		'-' indique un deplacement
		'?' indique une erreur de mnemo

*/
//"0123456789012345
//"n LII   n    .. "

char mnemo[] = "\
\x02 LII   n    \x00\x00 \
\x02 LIJ   n    \x00\x01 \
\x02 LIA   n    \x00\x02 \
\x02 LIB   n    \x00\x03 \
\x01 IX         \x00\x04 \
\x01 DX         \x00\x05 \
\x01 IY         \x00\x06 \
\x01 DY         \x00\x07 \
\x01 MVW        \x00\x08 \
\x01 EXW        \x00\x09 \
\x01 MVB        \x00\x0A \
\x01 EXB        \x00\x0B \
\x01 ADN        \x00\x0C \
\x01 SBN        \x00\x0D \
\x01 ADW        \x00\x0E \
\x01 SBW        \x00\x0F \
\x03 LIDP  &    \x00\x10 \
\x02 LIDL  n    \x00\x11 \
\x02 LIP   n    \x00\x12 \
\x02 LIQ   n    \x00\x13 \
\x01 ADB        \x00\x14 \
\x01 SBB        \x00\x15 \
\x02 INT   n    \x00\x16 \
\x01 ?          \x00\x17 \
\x01 MVWD       \x00\x18 \
\x01 EXWD       \x00\x19 \
\x01 MVBD       \x00\x1A \
\x01 EXBD       \x00\x1B \
\x01 SRW        \x00\x1C \
\x01 SLW        \x00\x1D \
\x01 FILM       \x00\x1E \
\x01 FILD       \x00\x1F \
\x01 LDP        \x00\x20 \
\x01 LDQ        \x00\x21 \
\x01 LDR        \x00\x22 \
\x01 RA         \x00\x23 \
\x01 IXL        \x00\x24 \
\x01 DXL        \x00\x25 \
\x01 IYS        \x00\x26 \
\x01 DYS        \x00\x27 \
\x02 JRNZP +    \x00\x28 \
\x02 JRNZM -    \x00\x29 \
\x02 JRNCP +    \x00\x2A \
\x02 JRNCM -    \x00\x2B \
\x02 JRP   +    \x00\x2C \
\x02 JRM   -    \x00\x2D \
\x01 ?          \x00\x2E \
\x02 LOOP  -    \x00\x2F \
\x01 STP        \x00\x30 \
\x01 STQ        \x00\x31 \
\x01 STR        \x00\x32 \
\x01  ?         \x00\x33 \
\x01 PUSH       \x00\x34 \
\x01 DATA       \x00\x35 \
\x01 ?          \x00\x36 \
\x01 RTN        \x00\x37 \
\x02 JRZP  +    \x00\x38 \
\x02 JRZM  -    \x00\x39 \
\x02 JRCP  +    \x00\x3A \
\x02 JRCM  -    \x00\x3B \
\x01 ?          \x00\x3C \
\x01 ?          \x00\x3D \
\x01 ?          \x00\x3E \
\x01 ?          \x00\x3F \
\x01 INCI       \x00\x40 \
\x01 DECI       \x00\x41 \
\x01 INCA       \x00\x42 \
\x01 DECA       \x00\x43 \
\x01 ADM        \x00\x44 \
\x01 SBM        \x00\x45 \
\x01 ANMA       \x00\x46 \
\x01 ORMA       \x00\x47 \
\x01 INCK       \x00\x48 \
\x01 DECK       \x00\x49 \
\x01 INCM       \x00\x4A \
\x01 DECM       \x00\x4B \
\x01 INA        \x00\x4C \
\x01 NOPW       \x00\x4D \
\x02 WAIT  n    \x00\x4E \
\x01 WAITI      \x00\x4F \
\x01 INCP       \x00\x50 \
\x01 DECP       \x00\x51 \
\x01 STD        \x00\x52 \
\x01 MVDM       \x00\x53 \
\x01 READM      \x00\x54 \
\x01 MVMD       \x00\x55 \
\x01 READ       \x00\x56 \
\x01 LDD        \x00\x57 \
\x01 SWP        \x00\x58 \
\x01 LDM        \x00\x59 \
\x01 SL         \x00\x5A \
\x01 POP        \x00\x5B \
\x01 ?          \x00\x5C \
\x01 OUTA       \x00\x5D \
\x01 ?          \x00\x5E \
\x01 OUTF       \x00\x5F \
\x02 ANIM  n    \x00\x60 \
\x02 ORIM  n    \x00\x61 \
\x02 TSIM  n    \x00\x62 \
\x02 CPIM  n    \x00\x63 \
\x02 ANIA  n    \x00\x64 \
\x02 ORIA  n    \x00\x65 \
\x02 TSIA  n    \x00\x66 \
\x02 CPIA  n    \x00\x67 \
\x01 ?          \x00\x68 \
\x01 DTJ        \x00\x69 \
\x01 ?          \x00\x6A \
\x02 TEST  n    \x00\x6B \
\x01 ?          \x00\x6C \
\x01 ?          \x00\x6D \
\x01 ?          \x00\x6E \
\x01 ?          \x00\x6F \
\x02 ADIM  n    \x00\x70 \
\x02 SBIM  n    \x00\x71 \
\x01 ?          \x00\x72 \
\x01 ?          \x00\x73 \
\x02 ADIA  n    \x00\x74 \
\x02 SBIA  n    \x00\x75 \
\x01 ?          \x00\x76 \
\x01 ?          \x00\x77 \
\x03 CALL  &    \x00\x78 \
\x03 JP    &    \x00\x79 \
\x04 PTJ   n    \x00\x7A \
\x01 ?          \x00\x7B \
\x03 JPNZ  &    \x00\x7C \
\x03 JPNC  &    \x00\x7D \
\x03 JPZ   &    \x00\x7E \
\x03 JPC   &    \x00\x7F \
\x01 LP    &00  \x00\x80 \
\x01 LP    &01  \x00\x81 \
\x01 LP    &02  \x00\x82 \
\x01 LP    &03  \x00\x83 \
\x01 LP    &04  \x00\x84 \
\x01 LP    &05  \x00\x85 \
\x01 LP    &06  \x00\x86 \
\x01 LP    &07  \x00\x87 \
\x01 LP    &08  \x00\x88 \
\x01 LP    &09  \x00\x89 \
\x01 LP    &0A  \x00\x8A \
\x01 LP    &0B  \x00\x8B \
\x01 LP    &0C  \x00\x8C \
\x01 LP    &0D  \x00\x8D \
\x01 LP    &0E  \x00\x8E \
\x01 LP    &0F  \x00\x8F \
\x01 LP    &10  \x00\x90 \
\x01 LP    &11  \x00\x91 \
\x01 LP    &12  \x00\x92 \
\x01 LP    &13  \x00\x93 \
\x01 LP    &14  \x00\x94 \
\x01 LP    &15  \x00\x95 \
\x01 LP    &16  \x00\x96 \
\x01 LP    &17  \x00\x97 \
\x01 LP    &18  \x00\x98 \
\x01 LP    &19  \x00\x99 \
\x01 LP    &1A  \x00\x9A \
\x01 LP    &1B  \x00\x9B \
\x01 LP    &1C  \x00\x9C \
\x01 LP    &1D  \x00\x9D \
\x01 LP    &1E  \x00\x9E \
\x01 LP    &1F  \x00\x9F \
\x01 LP    &20  \x00\xA0 \
\x01 LP    &21  \x00\xA1 \
\x01 LP    &22  \x00\xA2 \
\x01 LP    &23  \x00\xA3 \
\x01 LP    &24  \x00\xA4 \
\x01 LP    &25  \x00\xA5 \
\x01 LP    &26  \x00\xA6 \
\x01 LP    &27  \x00\xA7 \
\x01 LP    &28  \x00\xA8 \
\x01 LP    &29  \x00\xA9 \
\x01 LP    &2A  \x00\xAA \
\x01 LP    &2B  \x00\xAB \
\x01 LP    &2C  \x00\xAC \
\x01 LP    &2D  \x00\xAD \
\x01 LP    &2E  \x00\xAE \
\x01 LP    &2F  \x00\xAF \
\x01 LP    &30  \x00\xB0 \
\x01 LP    &31  \x00\xB1 \
\x01 LP    &32  \x00\xB2 \
\x01 LP    &33  \x00\xB3 \
\x01 LP    &34  \x00\xB4 \
\x01 LP    &35  \x00\xB5 \
\x01 LP    &36  \x00\xB6 \
\x01 LP    &37  \x00\xB7 \
\x01 LP    &38  \x00\xB8 \
\x01 LP    &39  \x00\xB9 \
\x01 LP    &3A  \x00\xBA \
\x01 LP    &3B  \x00\xBB \
\x01 LP    &3C  \x00\xBC \
\x01 LP    &3D  \x00\xBD \
\x01 LP    &3E  \x00\xBE \
\x01 LP    &3F  \x00\xBF \
\x01 INCJ       \x00\xC0 \
\x01 DECJ       \x00\xC1 \
\x01 INCB       \x00\xC2 \
\x01 DECB       \x00\xC3 \
\x01 ADCM       \x00\xC4 \
\x01 SBCM       \x00\xC5 \
\x01 TSMA       \x00\xC6 \
\x01 CPMA       \x00\xC7 \
\x01 INCL       \x00\xC8 \
\x01 DECL       \x00\xC9 \
\x01 INCN       \x00\xCA \
\x01 DECN       \x00\xCB \
\x01 INB        \x00\xCC \
\x01 ?          \x00\xCD \
\x01 NOPT       \x00\xCE \
\x01 ?          \x00\xCF \
\x01 SC         \x00\xD0 \
\x01 RC         \x00\xD1 \
\x01 SR         \x00\xD2 \
\x01 WRIT ??    \x00\xD3 \
\x02 ANID  n    \x00\xD4 \
\x02 ORID  n    \x00\xD5 \
\x02 TSID  n    \x00\xD6 \
\x01 ?          \x00\xD7 \
\x01 LEAVE      \x00\xD8 \
\x01 ?          \x00\xD9 \
\x01 EXAB       \x00\xDA \
\x01 EXAM       \x00\xDB \
\x01 ?          \x00\xDC \
\x01 OUTB       \x00\xDD \
\x01 ?          \x00\xDE \
\x01 OUTC       \x00\xDF \
\x02 CAL   &00m \x00\xE0 \
\x02 CAL   &01m \x00\xE1 \
\x02 CAL   &02m \x00\xE2 \
\x02 CAL   &03m \x00\xE3 \
\x02 CAL   &04m \x00\xE4 \
\x02 CAL   &05m \x00\xE5 \
\x02 CAL   &06m \x00\xE6 \
\x02 CAL   &07m \x00\xE7 \
\x02 CAL   &08m \x00\xE8 \
\x02 CAL   &09m \x00\xE9 \
\x02 CAL   &0Am \x00\xEA \
\x02 CAL   &0Bm \x00\xEB \
\x02 CAL   &0Cm \x00\xEC \
\x02 CAL   &0Dm \x00\xED \
\x02 CAL   &0Em \x00\xEE \
\x02 CAL   &0Fm \x00\xEF \
\x02 CAL   &10m \x00\xF0 \
\x02 CAL   &11m \x00\xF1 \
\x02 CAL   &12m \x00\xF2 \
\x02 CAL   &13m \x00\xF3 \
\x02 CAL   &14m \x00\xF4 \
\x02 CAL   &15m \x00\xF5 \
\x02 CAL   &16m \x00\xF6 \
\x02 CAL   &17m \x00\xF7 \
\x02 CAL   &18m \x00\xF8 \
\x02 CAL   &19m \x00\xF9 \
\x02 CAL   &1Am \x00\xFA \
\x02 CAL   &1Bm \x00\xFB \
\x02 CAL   &1Cm \x00\xFC \
\x02 CAL   &1Dm \x00\xFD \
\x02 CAL   &1Em \x00\xFE \
\x02 CAL   &1Fm \x00\xFF \
";

/****************************************************************************

    FUNCTION: Unassemble

    PURPOSE: Gnre un fichier de dessassemblage

****************************************************************************/
void unassemble( void)
{
    FILE *fpOut;  // fichier en sortie
    unsigned short i,j, adr;
    unsigned char k, oct1, oct2, oct3;


    char tmp[256];
	char lig[256];

    /* Ecriture du fichier RAM */
    if((fpOut=fopen(unassemble_file,"wb")) == NULL)
        error("Erreur d'ouverture du fichier unassemble");

	// securite, pour contrer les effets de bord
	if (unassemble_stop>=0xFFFE)
			unassemble_stop>=0xFFFD;

	if (unassemble_type != 0 )
	{
		// ORG &5C5A
		sprintf(lig,
		     "ORG &%04X ; Sets the start address of the program \n\nstart:\n",
		      unassemble_start );
		// ecriture de la ligne
		fwrite((char*)(lig),1,strlen(lig),fpOut);
	}
		/* signification des caracteres de type d'instruction
			'n' indique un octet
			'&' indique une adresse
			'+' indique un deplacement
			'-' indique un deplacement
			'?' indique une erreur de mnemo
		*/

	for (i=unassemble_start, k=0; i+k <= unassemble_stop; i+=  k)
    {
        // lecture d'un octet
        oct1 = peekPC( i );
        oct2 = peekPC( i+1 );
        oct3 = peekPC( i+1 );

		// exemple :
		//F101: 10 5F A2    LIDP  5FA2

        // recupere le mnemonique
        strcpy(tmp, &mnemo[16 * oct1] );

		// Recherche le caractere de type d'instruction
		for (j=12; j>5 ; j-- )
		{
			if (tmp[j] == 0x20)
			{
				tmp[j]= 0;
			}
			else
				break;
		}

		k= tmp[0];
		if (unassemble_type == 0 )
		{
			// Cas normal - presentation de type listing ex :
			// "5C5A: 10 5C 5C     "
	    	switch( k )
        	{
            case 1:
				sprintf(lig, "%04X: %02X          ",i, oct1);
                break;
	        case 2:
				sprintf(lig, "%04X: %02X %02X       ",i, oct1, oct2);
                break;
		    case 3:
				sprintf(lig, "%04X: %02X %02X %02X    ",i , oct1, oct2, oct3);
                break;
			}
			// ecriture du debut de la ligne
			fwrite((char*)(lig),1,strlen(lig),fpOut);

			// creation de la suite
			// Cas normal - presentation de type listing ex :
			// "LIDP  5C5C"
	        switch( tmp[j] )
	        {
	            case '?':
					tmp[j] = 0;
					sprintf(lig, "%s\n",&tmp[1]);
					break;
	            case 'm':
					tmp[j] = 0;
					sprintf(lig, "%s%02X\n",&tmp[1], oct2);
					break;
	            case 'n':
					tmp[j] = 0;
					sprintf(lig, "%s%02X\n",&tmp[1], oct2);
					break;
	            case '&':
					tmp[j] = 0;
					sprintf(lig, "%s%02X%02X\n",&tmp[1], oct2, oct3);
					break;
	            case '+':
					tmp[j] = 0;
					sprintf(lig, "%s%04X\n",&tmp[1],(unsigned short)(i+1+ oct2));
					break;
	            case '-':
					tmp[j] = 0;
					sprintf(lig, "%s%04X\n",&tmp[1],(unsigned short)(i+1- oct2));
					break;
	            default:
					sprintf(lig, "%s\n", &tmp[1]);
	                break;
    		}
	    	fwrite((char*)(lig),1,strlen(lig),fpOut);

		}
		else
		{
			// Cas special - presentation de type assembleur :
			// L5C5A: LIDP  &5C5C
	        switch( tmp[j] )
	        {
	            case '?':
					tmp[j] = 0;
					sprintf(lig, "L_%04X: %s\n",i, &tmp[1]);
					break;
	            case 'm':
					tmp[j] = 0;
					sprintf(lig, "L_%04X: %s%02X\n",i,&tmp[1], oct2);
					break;
	            case 'n':
					tmp[j] = 0;
					sprintf(lig, "L_%04X: %s&%02X\n",i,&tmp[1], oct2);
					break;
	            case '&':
					adr = (oct2<<8) + oct3;
					tmp[j] = 0;
					if ( (adr>= unassemble_start) && (adr<= unassemble_stop) )
					{
						sprintf(lig, "L_%04X: %sL_%02X%02X\n",i,&tmp[1], oct2, oct3);
					}
					else
					{
						sprintf(lig, "L_%04X: %s&%02X%02X\n",i,&tmp[1], oct2, oct3);
					}
					break;
	            case '+':
					tmp[j] = 0;
					adr = (unsigned short)(i+1+ oct2);
					if ( (adr>= unassemble_start) && (adr<= unassemble_stop) )
					{
						sprintf(lig, "L_%04X: %sL_%04X\n",i,&tmp[1],adr);
					}
					else
					{
						sprintf(lig, "L_%04X: %s&%04X\n",i,&tmp[1],adr);
					}
					break;
	            case '-':
					tmp[j] = 0;
					adr = (unsigned short)(i+1- oct2);
					if ( (adr>= unassemble_start) && (adr<= unassemble_stop) )
					{
						sprintf(lig, "L_%04X: %sL_%04X\n",i,&tmp[1],adr);
					}
					else
					{
						sprintf(lig, "L_%04X: %s&%04X\n",i,&tmp[1],adr);
					}
					break;
	            default:
					sprintf(lig, "L_%04X: %s\n",i, &tmp[1]);
	                break;
    		}
	    	fwrite((char*)(lig),1,strlen(lig),fpOut);
		}
	}

	if (unassemble_type != 0 )
	{
		sprintf(lig, "; End of program = &%04X\n",i);
		fwrite((char*)(lig),1,strlen(lig),fpOut);
	}
    fclose(fpOut);

}

